<?php

namespace LakeDrops\DorgFlow;

use Exception;
use GitElephant\Repository;
use LakeDrops\Component\Composer\BaseHandler;
use LakeDrops\Component\Dotenv\Dotenv;
use Symfony\Component\Filesystem\Filesystem;

/**
 * Class Handler.
 *
 * @package LakeDrops\DorgFlow
 */
class Handler extends BaseHandler {

  /**
   * Post install/update event to prepare projects for development.
   */
  public function prepareDevProjects() {

    // We only do the fancy stuff for developers.
    if (!$this->isDevMode() || $this->isCiContext()) {
      return;
    }

    $env = new Dotenv('dorgflow', $this->io);
    $dorgflow = $env->receiveGlobal('DORGFLOW', 'Dorgflow', '0');
    if (empty($dorgflow)) {
      return;
    }

    $installationManager = $this->composer->getInstallationManager();
    $options = $this->getOptions();
    foreach ([
               'projects' => 'git.drupal.org:project',
               'drupalspoons' => 'gitlab.com:drupalspoons',
             ] as $type => $url) {
      if (empty($options[$type])) {
        continue;
      }
      $this->io->write('Dorgflow: Preparing ' . $url . ' packages for development', TRUE);
      foreach ($options[$type] as $project => $version) {
        $package = $this->getPackage($project);
        if ($package === NULL) {
          continue;
        }

        $path = $installationManager->getInstallPath($package);
        $this->io->write("- $project => $path", TRUE);
        $this->prepareWorkingDirectory($path, $project, $version, $url);
      }
    }
  }

  /**
   * Retrieve options from composer.json "extra" configuration.
   *
   * @return array
   *   The options.
   */
  protected function getOptions(): array {
    $extra = $this->composer->getPackage()->getExtra() + ['dorgflow' => []];
    return $extra['dorgflow'] + [
        'projects' => [],
        'drupalspoons' => [],
      ];
  }

  /**
   * Prepare the working directory of a git based package for dorgflow.
   *
   * @param string $path
   *   Name of the working directory.
   * @param string $project
   *   Name of the project.
   * @param string $version
   *   Version to checkout.
   * @param string $url
   *   Base URL to checkout from.
   */
  protected function prepareWorkingDirectory($path, $project, $version, $url) {
    [, $projectname] = explode('/', $project);
    $uri = 'git@' . $url . '/' . $projectname . '.git';

    // Git Clone the repository into the working directory.
    $repository = Repository::open($path);
    $repository->init();

    try {
      $origin = $repository->getRemote('origin', FALSE);
      if ($origin && $origin->getFetchURL() === $uri) {
        // Already setup correctly.
        return;
      }
    }
    catch (Exception $ex) {
      // Ignore the exception and conitue setup.
    }

    $fs = new Filesystem();
    $fs->remove($path);
    $fs->mkdir($path);
    $repository->init();
    $repository->addRemote('origin', $uri);

    $repository->fetch();
    $repository->checkout($version);
    $repository->getCaller()->execute('branch --set-upstream-to origin/' . $version . ' ' . $version);
    $this->io->write('  - completed', TRUE);
  }

}
